#!/usr/bin/env python

import collections
import json
import re
from pathlib import Path


def get_program_parameters():
    import argparse
    description = 'Generate a list of vtk modules and classes for Python.'
    epilogue = '''
Use modules.json and your source file listing all the vtk modules and classes
  needed by the python source.

Note than if there are spaces in the paths, enclose the path in quotes.

    '''
    parser = argparse.ArgumentParser(description=description, epilog=epilogue,
                                     formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('-j', '--json', default=['modules.json'],
                        help='The path to the VTK JSON file (modules.json).')
    parser.add_argument('-s', '--source', nargs='+', help='The path to the python source file.')
    args = parser.parse_args()
    return args.json, args.source


def build_classes_modules(jpath):
    """
    From the parsed JSON data file make a dictionary whose key is the
     header filename and value is the module.

    :param jpath: The JSON file path.
    :return: VTK Classes and modules.
    """
    with open(jpath) as data_file:
        json_data = json.load(data_file)

    # The headers should be unique to a module, however we will not assume this.
    res = dict()
    for k, v in json_data['modules'].items():
        if 'headers' in v:
            for k1 in v['headers']:
                # Assuming the header name corresponds to the class name.
                res[k1.split('.')[0]] = k
    return res


def get_vtk_classes(path):
    """
    Extract the vtk class names from the path.

    :param path: The path to the Python file.
    :return: The file name and the vtk classes.
    """
    res = collections.defaultdict(set)
    content = path.read_text().split('\n')
    for line in content:
        vtk_class_pattern = re.compile(r'(vtk[a-zA-Z]+\S+\(\))')
        vtk_class_pattern1 = re.compile(r'[^\(\)]+')
        m = vtk_class_pattern.search(line)
        if m:
            for g in m.groups():
                m1 = vtk_class_pattern1.match(g)
                res[path.stem].add(m1.group())

    return res


def display_imports(imports):
    name_keys = sorted(imports.keys())
    res = None
    for name in name_keys:
        res = f'{name}\n'
        module_keys = sorted(imports[name].keys())
        for module in module_keys:
            classes = sorted(list(imports[name][module]))
            m = module.split('::')
            if len(m) > 1 and m[0] == 'VTK':
                if len(classes) == 1:
                    res += f'from vtkmodules.vtk{m[1]} import {classes[0]}\n'
                else:
                    c_list = list()
                    for c in classes:
                        c_list.append(f'    {c}')
                    s = '(\n'
                    s += ',\n'.join(c_list)
                    s += '\n    )'
                    res += f'from vtkmodules.vtk{m[1]} import {s}\n'
    return res


def main():
    json_path, src_paths = get_program_parameters()

    # json_paths always has a default value.
    # Assume everything we need is in modules.json.
    jpath = Path(json_path)
    if jpath.is_dir():
        jpath = jpath / 'modules.json'
    if not jpath.is_file():
        print(f'Non existent JSON file: {jpath}')

    assert src_paths is not None, 'Enter your Python file name(s).'

    imports = collections.defaultdict(lambda: collections.defaultdict(set))
    for fn in src_paths:
        path = Path(fn)
        if path.is_file() and path.suffix == '.py':

            vtk_classes = get_vtk_classes(path)
        else:
            print(f'Either not a file or wrong extension: {fn}')

    classes_modules = build_classes_modules(jpath)
    for name, vtk_classes in vtk_classes.items():
        for vtk_class in vtk_classes:
            if vtk_class in classes_modules:
                module = classes_modules[vtk_class]
                imports[name][module].add(vtk_class)

    res = display_imports(imports)
    print(res)


if __name__ == '__main__':
    main()
